/*
 *Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and conditions in the IBM International Program License Agreement.
 *
 * Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package superscript;

import com.rational.test.ft.object.interfaces.*;
import com.rational.test.ft.script.*;
import com.rational.test.ft.value.*;
import com.rational.test.ft.vp.*;
import java.util.*;

/**
 * Provides some general utility methods.
 */
abstract public class ExtensionScript extends RationalTestScript 
{
	/** 
	 * Make a beep sound on the system speaker.
	 */
	public static void beep()
	{
        java.awt.Toolkit.getDefaultToolkit().beep();
	}
	
	/**
	 * return the text on the clipboard.
	 * If there is no text, it returns null
	 */
	public static String getClipboardText()
	{
        java.awt.datatransfer.Clipboard clip = java.awt.Toolkit.getDefaultToolkit().getSystemClipboard();
        java.awt.datatransfer.Transferable t = clip.getContents(null);
        String s = null;
        try
        {
	       	s = (String)t.getTransferData(java.awt.datatransfer.DataFlavor.stringFlavor);
        } 
        catch(Exception e) {}
        return s;
	}
	
	/** 
	 * Set the contents of the clipboard to the provided text
	 */
	public static void setClipboardText(String s)
	{
        java.awt.datatransfer.Clipboard clip = java.awt.Toolkit.getDefaultToolkit().getSystemClipboard();
  		java.awt.datatransfer.StringSelection ss = new java.awt.datatransfer.StringSelection(s);
		clip.setContents(ss, ss);
	}

	/**
	 * A simple text vp for the clipboard.  If you insert a call to this method
	 * in your test script, the first time the script is run, the clipboard text will
	 * be captured and written to a VP baseline named by <code>vpName</code>. After
	 * the baseline is created on the first playback, running the script will cause the
	 * clipboard text to be captured, compared to the baseline, and the result will
	 * be written to the test log.
	 */
	public void clipboardVP(String vpName)
	{
		vpManual(vpName, getClipboardText()).performTest();
	}

	
	/**
	 * A simple timer class - it can be called in a polling loop to determine
	 * when your time is up.
	 */
	public static class Timer
	{
		private long start = 0;
		
		/**
		 * constructor - starts the timer.
		 */
		public Timer()
		{
			reset();
		}
	
		/**
		 * Reset the timer to the current time.
		 */
		public void reset()
		{
			start = System.currentTimeMillis();
		}
		
		/**
		 * Returns elapsed time (since timer was created) in seconds
		 */
		public double elapsedTime()
		{
			long end = System.currentTimeMillis();
			return (end - start)/1000;
		}
		
		/**
		 * Returns true if elapsed time (since timer was created) 
		 * is greater than seconds
		 */
		public boolean isTimeUp(double seconds)
		{
			return elapsedTime() > seconds;
		}
	}
	
	/**
	 * Wait for an object with default max time and delay between retries. 
	 * The finder interface is called to determine if the object is yet found.
	 */
	public Object waitForObject(IFindObject finder)
	{
		// use the standard option delay times
		double maxTime = ((Double)getOption(IOptionName.MAXIMUM_FIND_OBJECT_TIME)).doubleValue();
		double delay = ((Double)getOption(IOptionName.FIND_OBJECT_DELAY_BETWEEN_RETRIES)).doubleValue();

		return waitForObject(finder, maxTime, delay);
	}	
	/**
	 * Wait for an object with specified max time and delay between retries.
	 * As this method uses the RationalTestScript.sleep() method, it can be
	 * interrupted by the user hitting {F11}.
	 * The finder interface is called to determine if the object is yet found.
	 */
	public Object waitForObject(IFindObject finder, double maxTime, double delay)
	{

		Timer t = new Timer();
		while(!t.isTimeUp(maxTime))
		{
			Object obj = finder.findObject();
			if (obj != null)
			{
				return obj;
			}
			
			sleep(delay);
		}
		throw new ExtensionScriptException("waitForObject timed out: "+finder.getClass().getName());
	}
	
	/**
	 * Wait for a vp with specified max time and delay between retries.
	 * As this method uses the RationalTestScript.sleep() method, it can be
	 * interrupted by the user hitting {F11}.
	 * The IFtVerificationPoint.compare() method is called to determine if the VP passes.
	 * Setting the compareTrueEqualsPass parameter to false can reverse the meaning of the VP 
	 * The returned object contains the stored results. If you then invokes compare()
	 * on the returned object, you will get the boolean result (pass or fail).  If you
	 * invoke performTest() on the returned object, the final result will be logged. Note
	 * that the returned object will not be affected by
	 * the compareTrueEqualsPass parameter.  To invert the logging, you must
	 * pass it in to performTest() as there is a version of performTest that
	 * takes the same parameter. 
	 */
	public IFtVerificationPoint waitForVp(IFtVerificationPoint vp, boolean compareTrueEqualsPass, double maxTime, double delay)
	{

		Timer t = new Timer();
		while(!t.isTimeUp(maxTime))
		{
			// internal method - this will be supported on IFtVerificationPoint
			// in a future release
			((com.rational.test.ft.vp.impl.FtVerificationPoint)vp).capture();
			boolean b = vp.compare();
			if (b == compareTrueEqualsPass)
			{
				return vp;
			}
			
			sleep(delay);
		}
		return vp;
	}
	
	/**
	 * Wait for an VP with default max time and delay between retries. 
	 * The IFtVerificationPoint.compare() method is called to determine if the VP passes.
	 * Setting the compareTrueEqualsPass parameter to false can reverse the meaning of the VP.
	 */
 	public IFtVerificationPoint waitForVp(IFtVerificationPoint vp, boolean compareTrueEqualsPass )
	{
		// use the standard option delay times
		double maxTime = ((Double)getOption(IOptionName.MAXIMUM_FIND_OBJECT_TIME)).doubleValue();
		double delay = ((Double)getOption(IOptionName.FIND_OBJECT_DELAY_BETWEEN_RETRIES)).doubleValue();

		return waitForVp(vp, compareTrueEqualsPass , maxTime, delay);
	}	
	
	/**
	 * Wait for an VP with default max time and delay between retries. 
	 * The IFtVerificationPoint.compare() method is called to determine if the VP passes.
	 */
	public IFtVerificationPoint waitForVp(IFtVerificationPoint vp)
	{
		return waitForVp(vp, true);
	}	
	
	/**
	 * Wait for an VP with default max time and delay between retries. 
	 * The IFtVerificationPoint.compare() method is called to determine if the VP passes.
	 */
	public IFtVerificationPoint waitForVpFail(IFtVerificationPoint vp)
	{
		return waitForVp(vp, false);
	}	
	
	/**
	 * interface definition for an object finder. 
	 */
	public interface IFindObject
	{
		/**
		 * returns an object if it can be found, returns null when no
		 * object is found
		 */
		public Object findObject();
	}
	
	/**
	 * pretty-print a hashtable (sorted)
	 */	
	public static void printHash(Hashtable h)
	{
		TreeMap sortedMap = new TreeMap(h);
		Set keySet = sortedMap.keySet();
		Iterator iter = keySet.iterator();
		
		while( iter.hasNext())
		{
			Object key = iter.next();
			Object value = h.get(key);
			System.out.println(key + " = " + value);
			
		}
	}

	/**
	 * print out the methods of an object (omitting methods from java.lang.Object)
	 */	
	public static void printMethods(TestObject obj)
	{
		MethodInfo[] m = obj.getMethods();
		for (int i = 0; i < m.length; ++i)
		{
			if (!m[i].getDeclaringClass().equals("java.lang.Object"))
				System.out.println( m[i] );
		}
	}

	/**
	 * General-purpose exception for extension scripts
	 */
	public static class ExtensionScriptException extends RuntimeException
	{
		ExtensionScriptException(String s) { super(s); }
	}
	
	/**
	 * print the properties of a TestObject.
	 */
	public void printProperties( TestObject to )
	{
		java.util.Hashtable table = to.getProperties();

		java.util.Enumeration keys = table.keys();

		while( keys.hasMoreElements() )
		{
			Object key = keys.nextElement();

			System.out.println( "Property " + key.toString() + " = " + table.get( key ) );
		}	
	}

	/**
	 * print the properties of a TestObject as an array.
	 */
	public void printPropertiesAsArray( TestObject to )
	{
		java.util.Hashtable table = to.getProperties();

		java.util.Enumeration keys = table.keys();

		int count = 0;

		while( keys.hasMoreElements() )
		{
			Object key = keys.nextElement();

			System.out.println( "\"" + key.toString() + "\"," );
		}			
	}
	
}


