/*
 *Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and conditions in the IBM International Program License Agreement.
 *
 * Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package superscript;

import com.rational.test.ft.ObjectNotFoundException;
import com.rational.test.ft.object.interfaces.BrowserTestObject;
import com.rational.test.ft.object.interfaces.DomainTestObject;
import com.rational.test.ft.object.interfaces.IGraphical;
import com.rational.test.ft.object.interfaces.ProcessTestObject;
import com.rational.test.ft.object.interfaces.TestObject;
import com.rational.test.ft.object.interfaces.TopLevelTestObject;
import com.rational.test.ft.script.IMapPropertyName;
import com.rational.test.ft.script.ITestObjectMethodState;


/**
 * This class provides some base capabilities for working
 * with HTML.
 * 
 * Any script that extends this will handle onObjectNotFound by
 * trying to dismiss a dialog box belonging to the browser and retrying the
 * original action.
 */
public abstract class HtmlScript extends ExtensionScript 
{
	/**
	 * Overrides the base implementation of onObjectNotFound.  Whenever
	 * this event occurs, we look through all the active domains (places
	 * where we might find objects).  For Html domains (we skip over Java 
	 * and other domains) we find all the top objects.  If the top object
	 * is an Html Dialog, we type an enter key at it to dismiss it.
	 * We log a warning when this happens.
	 */
	public void onObjectNotFound(ITestObjectMethodState testObjectMethodState)
	{
		if (!testObjectMethodState.getTestObject().
				getPropertyFromMap(IMapPropertyName.DOMAIN)
				.equals("Html"))
		{
			return;
		}
		boolean dismissedAWindow = false;
		DomainTestObject domains[] = getDomains();
		for (int i = 0; i < domains.length; ++i)
		{
			if (domains[i].getName().equals("Html"))
			{
				// We found an Html domain.
				TestObject[] topObjects = domains[i].getTopObjects();
				if (topObjects != null)
				{
					try
					{
						for (int j = 0; j < topObjects.length; ++j)
						{
							if (topObjects[j].getProperty(".class").equals("Html.Dialog"))
							{
								// We found a top-level HtmlDialog.
								logWarning("HtmlScript.onObjectNotFound - dismissing Html Dialog");
								try
								{
									dismissedAWindow = true;
									((TopLevelTestObject)topObjects[j]).inputKeys("{enter}");
									
								}
								catch(RuntimeException e) {}
							}
						}
					}
					finally
					{
						// unregister all references to top objects
						unregister(topObjects);
					}
				}
			}
		}
		if (dismissedAWindow)
		{
			// try again - we may have unblocked the problem!
			testObjectMethodState.findObjectAgain();
		}
		else
		{
			logWarning("HtmlScript.onObjectNotFound - found no Html Dialog to dismiss");
		}
	}

	public void closeBrowser(BrowserTestObject testObject)
	{
		String sName = (String)testObject.getProperty(".browserName");
		testObject.close();								   
		if(!sName.equals ("MS Internet Explorer")) 
		{
			ProcessTestObject proc = testObject.getProcess();
			while(proc.isAlive())
				sleep(1);
		}
	}


	/**
	 * Find the HTML node with the specified text.  If no child
	 * node with the specified text is located then an 
	 * <code>ObjectNotFoundException</code> exception is thrown.
	 * 
	 * @param	parent	The root node for a search, typically a page 
	 * 					reference.
	 * @param	text	The text on the desired child node.  The text
	 * 					is compared in a case insensitive fashion.
	 * @return	The node with the desired text.  An exception is thrown
	 * 			if the node is not found.
	 */
	public IGraphical findHtmlChildWithText(TestObject parent, String text)
	{
		IGraphical result = findHtmlChildWithTextInner(parent, text);
		if ( result == null ) // throw exception so check for null not needed in script
			throw new ObjectNotFoundException("Unable to find child with text: "+text);
		return result;
	}
	
	private IGraphical findHtmlChildWithTextInner(TestObject parent, String text)
	{
		if ( parent instanceof IGraphical )
		{
			Object prop = parent.getProperty(".text");
			if ( prop != null && text.equalsIgnoreCase(prop.toString()) )
				return (IGraphical)parent;
		}
		TestObject[] children = parent.getMappableChildren();
		int length = ( children != null ? children.length : 0 );
		for ( int i = 0; i < length; ++i )
		{
			IGraphical result = findHtmlChildWithTextInner(children[i], text);
			if ( result != null )
				return result;
		}
		return null;
	}

}

