/*
 */
package superscript;

import com.rational.test.ft.object.interfaces.*;
import com.rational.test.util.regex.Regex;

/**
 */
public class NativeWindowScript extends FileSupportScript
{

	/**
	 * Close the active window by clicking the close icon on the 
	 * windows titlebar.
	 */
	protected void closeActiveWindow()
	{
		IWindow activeWindow = getScreen().getActiveWindow();
		if ( activeWindow != null )
			activeWindow.close();
	}

	/**
	 * Pass an {ENTER} key code to the active window.  This will typically
	 * trigger the default button with focus (commonly the "OK" or "Cancel"
	 * button).
	 */
	protected void acceptActiveWindow()
	{
		IWindow activeWindow = getScreen().getActiveWindow();
		if ( activeWindow != null )
			activeWindow.inputKeys("{ENTER}");
	}
	
	/**
	 * Given the expected caption find a top level window that has that caption.
	 * The first window found with the caption is returned.
	 * 
	 * @param caption The caption of the desired window.
	 * @param ignoreCase Flags if case i significant in the caption comparisons.
	 * @return The first top level window with the specified text that is located.
	 */
	protected IWindow getTopWindowWithCaption(String caption, boolean ignoreCase)
	{
		if ( caption == null )
			caption = "";
		IWindow[] topWindows = getTopWindows();
		int length = ( topWindows != null ? topWindows.length : 0 );
		for ( int i = 0; i < length; ++i )
		{
			String text = topWindows[i].getText();
			if ( ( !ignoreCase && caption.equals(text) ) ||
				 ( ignoreCase && caption.equalsIgnoreCase(text) ) )
				return topWindows[i];
		}
		return null;
	}
	
	/**
	 * Given the expected caption pattern find all top level windows that have a caption
	 * that matches the pattern.  Basically if the pattern is "^Notepad -" all top level
	 * windows that have a caption that starts with "Notepad -" will be returned.
	 * 
	 * @param pattern The caption pattern of the desired windows.
	 * @param ignoreCase Flags if case is significant in the caption pattern matches.
	 * @return The set top level windows with the specified caption text pattern.
	 */
	protected IWindow[] getTopWindowsWithCaptionPattern(String pattern, boolean ignoreCase)
	{
		Regex regex = ( ignoreCase ? new Regex(pattern, Regex.MATCH_CASEINDEPENDENT) :
									 new Regex(pattern) );
		java.util.Vector matches = new java.util.Vector(10);
		IWindow[] topWindows = getTopWindows();
		int length = ( topWindows != null ? topWindows.length : 0 );
		for ( int i = 0; i < length; ++i )
		{
			String text = topWindows[i].getText();
			if ( regex.matches(text) )
				matches.add(topWindows[i]);
		}
		
		int resultLength = matches.size();
		IWindow[] result = ( resultLength != 0 ?
							 new IWindow[resultLength] :
							 null );
		for ( int i = 0; i < resultLength; ++i )
			result[i] = (IWindow)matches.elementAt(i);
		return result;
	}
	
	/**
	 * Given a particular context, find a child with the specified text.
	 * Typically this is used to locate a pushbutton with specific text
	 * on it, such as "OK" or "Cancel".
	 * 
	 * @param parent The context from which the search is based, typically
	 * 				 a top level window like a dialog box.
	 * @param text	The text on the window being sought.
	 * @param ignoreCase <code>true</code> if the case of the text should not
	 *  				 be taken into consideration.
	 * @param onlyImmediateChildren <code>true</code> if the search should be 
	 *                              limited to only the children of the parent
	 *                              window and not recurse into the grandchildren.
	 * @return The control with the specified text or <code>null</code> if no
	 *         child with the text is located.
	 */
	protected IWindow getChildWindowWithText(IWindow parent, 
											 String text, 
											 boolean ignoreCase, 
											 boolean onlyImmediateChildren)
	{
		if ( text == null )
			return null;
		IWindow[] children = ( parent != null ? parent.getChildren() : null );
		int length = ( children != null ? children.length : 0 );
		for ( int i = 0; i < length; ++i )
		{
			IWindow child = children[i];
			String childText = child.getText();
			if ( ( !ignoreCase && text.equals(childText) ) ||
				 ( ignoreCase && text.equalsIgnoreCase(childText) ) )
				return child;
			if ( !onlyImmediateChildren )
			{
				IWindow grandchild = getChildWindowWithText(child, text, ignoreCase, onlyImmediateChildren);
				if ( grandchild != null )
					return grandchild;
			}
		}
		return null;
	}

	/**
	 * Given a particular context, find a child with text that matches the
	 * specified regular expression pattern.
	 * Typically this is used to locate a pushbutton with specific text
	 * on it, such as "OK" or "Cancel" when the format of the text is not specific
	 * (for instance when the text equals " OK " instead of a simple "OK").
	 * 
	 * @param parent The context from which the search is based, typically
	 * 				 a top level window like a dialog box.
	 * @param pattern	The pattern for the text on the window being sought.
	 * @param ignoreCase <code>true</code> if the case of the text should not
	 *  				 be taken into consideration.
	 * @param onlyImmediateChildren <code>true</code> if the search should be 
	 *                              limited to only the children of the parent
	 *                              window and not recurse into the grandchildren.
	 * @return The control with the specified text or <code>null</code> if no
	 *         child with the text is located.
	 */
	protected IWindow getChildWindowWithTextPattern(IWindow parent, 
											 		String pattern, 
											 		boolean ignoreCase, 
											 		boolean onlyImmediateChildren)
	{
		Regex regex = ( ignoreCase ? new Regex(pattern, Regex.MATCH_CASEINDEPENDENT) :
									 new Regex(pattern) );
		IWindow[] children = ( parent != null ? parent.getChildren() : null );
		int length = ( children != null ? children.length : 0 );
		for ( int i = 0; i < length; ++i )
		{
			IWindow child = children[i];
			String childText = child.getText();
			if ( regex.matches(childText) )
				return child;
			if ( !onlyImmediateChildren )
			{
				IWindow grandchild = getChildWindowWithText(child, pattern, ignoreCase, false);
				if ( grandchild != null )
					return grandchild;
			}
		}
		return null;
	}

}
